@@ -28,13 +28,28 @@ from thoth.python import PackageVersion
 from thoth.python import Source
 from thoth.storages import GraphDatabase
 
-from ..base import AdviserTestCase
+from ..base import AdviserUnitTestCase
 
 
-class TestSecurityIndicatorStep(AdviserTestCase):
+class TestSecurityIndicatorStep(AdviserUnitTestCase):
     """Test different aspects of si pipeline step."""
 
+    UNIT_TESTED = SecurityIndicatorStep
+
     _SECURITY_INFO_EXISTS = {
+        "severity_high_confidence_high": 0,
+        "severity_high_confidence_medium": 5,
+        "severity_high_confidence_low": 3,
+        "severity_medium_confidence_high": 2,
+        "severity_medium_confidence_medium": 0,
+        "severity_medium_confidence_low": 8,
+        "severity_low_confidence_high": 21,
+        "severity_low_confidence_medium": 6,
+        "severity_low_confidence_low": 0,
+        "number_of_lines_with_code_in_python_files": 692,
+    }
+
+    _HIGH_HIGH_SECURITY_INFO = {
         "severity_high_confidence_high": 1,
         "severity_high_confidence_medium": 5,
         "severity_high_confidence_low": 3,
@@ -47,6 +62,11 @@ class TestSecurityIndicatorStep(AdviserTestCase):
         "number_of_lines_with_code_in_python_files": 692,
     }
 
+    def test_verify_multiple_should_include(self, builder_context: PipelineBuilderContext) -> None:
+        """Verify multiple should_include calls do not loop endlessly."""
+        builder_context.recommendation_type = RecommendationType.SECURITY
+        self.verify_multiple_should_include(builder_context)
+
     @pytest.mark.parametrize("recommendation_type", [RecommendationType.STABLE, RecommendationType.SECURITY])
     def test_include(self, builder_context: PipelineBuilderContext, recommendation_type: RecommendationType) -> None:
         """Test including this pipeline unit."""
@@ -76,6 +96,7 @@ class TestSecurityIndicatorStep(AdviserTestCase):
         )
 
         context = flexmock(graph=GraphDatabase())
+        context.recommendation_type = RecommendationType.STABLE
         with SecurityIndicatorStep.assigned_context(context):
             step = SecurityIndicatorStep()
             result = step.run(None, package_version)
@@ -132,3 +153,21 @@ class TestSecurityIndicatorStep(AdviserTestCase):
             result[1][0]["message"] == "flask===0.12.0 on https://pypi.org/simple has no "
             "gathered information regarding security."
         )
+
+    def test_security_indicator_with_high_confidence(self) -> None:
+        """Make sure we don't accept package if si info is missing when recommendation is secure."""
+        flexmock(GraphDatabase)
+        GraphDatabase.should_receive("get_si_aggregated_python_package_version").with_args(
+            package_name="flask", package_version="0.12.0", index_url="https://pypi.org/simple"
+        ).and_return(self._HIGH_HIGH_SECURITY_INFO).once()
+
+        package_version = PackageVersion(
+            name="flask", version="==0.12.0", index=Source("https://pypi.org/simple"), develop=False,
+        )
+
+        context = flexmock(graph=GraphDatabase())
+        context.recommendation_type = RecommendationType.SECURITY
+        with pytest.raises(NotAcceptable):
+            with SecurityIndicatorStep.assigned_context(context):
+                step = SecurityIndicatorStep()
+                step.run(None, package_version)
