@@ -17,6 +17,7 @@
 
 """Pseudonym units implemented in adviser."""
 
+from ._debug import AliasPseudonym
 from .intel_tensorflow import IntelTensorFlowPseudonym
 from .tensorflow_gpu import TensorFlowGPUPseudonym
 
@@ -24,6 +25,7 @@ from .tensorflow_gpu import TensorFlowGPUPseudonym
 # in which the asked to be registered - any dependencies between them
 # can be mentioned here.
 __all__ = [
+    "AliasPseudonym",
     "IntelTensorFlowPseudonym",
     "TensorFlowGPUPseudonym",
 ]
