@@ -29,6 +29,8 @@ import logging
 import attr
 from thoth.common import get_justification_link as jl
 from thoth.python import PackageVersion
+from voluptuous import Schema
+from voluptuous import Required
 
 from ..enums import RecommendationType
 from ..exceptions import NotAcceptable
@@ -50,20 +52,31 @@ class SecurityIndicatorStep(Step):
     _logged_packages = attr.ib(type=Set[Tuple[str, str, str]], default=attr.Factory(set), init=False)
 
     CONFIGURATION_DEFAULT = {
-        "high_confidence_weight": 1,
-        "medium_confidence_weight": 0.5,
+        "high_confidence_weight": 1.0,
+        "high_severity_weight": 100.0,
         "low_confidence_weight": 0.1,
-        "high_severity_weight": 100,
-        "medium_severity_weight": 10,
-        "low_severity_weight": 1,
+        "low_severity_weight": 1.0,
+        "medium_confidence_weight": 0.5,
+        "medium_severity_weight": 10.0,
+        "multi_package_resolution": False,
+        "package_name": None,
         "si_reward_weight": 0.5,
     }
+    CONFIGURATION_SCHEMA: Schema = Schema(
+        {
+            Required("high_confidence_weight"): float,
+            Required("high_severity_weight"): float,
+            Required("low_confidence_weight"): float,
+            Required("low_severity_weight"): float,
+            Required("medium_confidence_weight"): float,
+            Required("medium_severity_weight"): float,
+            Required("multi_package_resolution"): bool,
+            Required("package_name"): None,
+            Required("si_reward_weight"): float,
+        }
+    )
     _JUSTIFICATION_LINK = jl("security")
 
-    def pre_run(self) -> None:
-        """Initialize this pipeline step before running the pipeline."""
-        self._logged_packages.clear()
-
     @classmethod
     def should_include(cls, builder_context: "PipelineBuilderContext") -> Optional[Dict[str, Any]]:
         """Register only if we are explicitly recommending secure stacks."""
@@ -85,6 +98,11 @@ class SecurityIndicatorStep(Step):
             }
         ]
 
+    def pre_run(self) -> None:
+        """Initialize this pipeline step before running the pipeline."""
+        self._logged_packages.clear()
+        super().pre_run()
+
     def run(
         self, state: State, package_version: PackageVersion
     ) -> Optional[Tuple[Optional[float], Optional[List[Dict[str, str]]]]]:
@@ -95,9 +113,10 @@ class SecurityIndicatorStep(Step):
             index_url=package_version.index.url,
         )
 
+        package_version_tuple = package_version.to_tuple_locked()
+
         if s_info is None:
             if self.context.recommendation_type == RecommendationType.SECURITY:
-                package_version_tuple = package_version.to_tuple_locked()
                 if package_version_tuple not in self._logged_packages:
                     self._logged_packages.add(package_version_tuple)
                     _LOGGER.warning(
@@ -114,6 +133,18 @@ class SecurityIndicatorStep(Step):
                 ),
             )
 
+        if (
+            self.context.recommendation_type == RecommendationType.SECURITY
+            and s_info["severity_high_confidence_high"] != 0
+        ):
+            if package_version_tuple not in self._logged_packages:
+                self._logged_packages.add(package_version_tuple)
+                _LOGGER.warning(
+                    "Skipping including package %r because bandit found high security high confidence issue(s).",
+                    package_version_tuple,
+                )
+            raise NotAcceptable
+
         s_score = (
             (
                 s_info["severity_high_confidence_high"] * self.configuration["high_confidence_weight"]
